@@ -5,16 +5,50 @@
  **********************************************************************
  */
 #include <stdio.h>
+#include <string.h>
+
 #include "sieve.h"
 #include "unicode/utimer.h"
 #include "udbgutil.h"
+#include "unicode/ustring.h"
 #include "unicode/decimfmt.h"
+#include "unicode/udat.h"
+
+#if U_PLATFORM_IMPLEMENTS_POSIX
+#include <unistd.h>
+
+static void usage(const char *prog) {
+  fprintf(stderr, "Usage: %s [ -f outfile.xml ] [ -t 'TestName' ]\n", prog);
+}
+#endif
+
 void runTests(void);
 
+#ifndef ITERATIONS
+#define ITERATIONS 5
+#endif
+
+
 FILE *out = NULL;
 UErrorCode setupStatus = U_ZERO_ERROR;
+const char *outName = NULL;
+int listmode = 0;
+const char *testName = NULL;
+const char *progname = NULL;
+int errflg = 0;
+int testhit = 0;
 
-int main(int argc, const char* argv[]){
+int testMatch(const char *aName) {
+  if(testName==NULL) return 1;
+  int len = strlen(testName);
+  if(testName[len-1]=='*') {
+    return strncmp(testName,aName,len-1);
+  } else {
+    return strcmp(testName,aName);
+  }
+}
+
+int main(int argc, char * const * argv){
 #if U_DEBUG
   fprintf(stderr,"%s: warning: U_DEBUG is on.\n", argv[0]);
 #endif
@@ -26,25 +60,74 @@ int main(int argc, const char* argv[]){
   }
 #endif
 
+#if U_PLATFORM_IMPLEMENTS_POSIX
+  int c;
+  extern int optind;
+  extern char *optarg;
+  while((c=getopt(argc,argv,"lf:t:")) != EOF) {
+    switch(c) {
+    case 'f':
+      outName = optarg;
+      break;
+    case 'l':
+      listmode++;
+      break;
+    case 't':
+      testName = optarg;
+      break;
+    case '?':
+      errflg++;
+    }
+    if(errflg) {
+      usage(progname);
+      return 0;
+    }
+  }
+  /* for ( ; optind < argc; optind++) {     ... argv[optind] } */
+#else
   if(argc==2) {
-    out=fopen(argv[1],"w");
+    outName = argv[1];
+  } else if(argc>2) {
+    fprintf(stderr, "Err: usage: %s [ output-file.xml ]\n", argv[0]);
+  }
+#endif
+
+    if(listmode && outName != NULL ) {
+      fprintf(stderr, "Warning: no output when list mode\n");
+      outName=NULL;
+    }
+
+  if(outName != NULL) {
+
+
+    out=fopen(outName,"w");
     if(out==NULL) {
-      fprintf(stderr,"Err: can't open %s for writing.\n", argv[1]);
+      fprintf(stderr,"Err: can't open %s for writing.\n", outName);
       return 1;
+    } else {
+      fprintf(stderr, "# writing results to %s\n", outName);
     }
     fprintf(out, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
     fprintf(out, "<tests icu=\"%s\">\n", U_ICU_VERSION);
     fprintf(out, "<!-- %s -->\n", U_COPYRIGHT_STRING);
-  } else if(argc>2) {
-    fprintf(stderr, "Err: usage: %s [ output-file.xml ]\n", argv[0]);
+  } else {
+    fprintf(stderr, "# (no output)\n");
+  }
+  
+  if(listmode && testName!=NULL) {
+    fprintf(stderr, "ERR: no -l mode when specific test with -t\n");
+    usage(progname);
     return 1;
   }
 
+
   runTests();
   
 
   if(out!=NULL) {
+#ifndef SKIP_INFO
     udbg_writeIcuInfo(out);
+#endif
     fprintf(out, "</tests>\n");
     fclose(out);
   }
@@ -68,6 +151,8 @@ protected:
    */
   virtual int32_t run() = 0;
   virtual void warmup() {  run(); } 
+public:
+  virtual const char *getName() { return fName; }
 public:
   virtual int32_t runTest(double *subTime) {
     UTimer a,b;
@@ -80,7 +165,6 @@ public:
 
   virtual int32_t runTests(double *subTime, double *marginOfError) {
     warmup(); /* warmup */
-    #define ITERATIONS 5
     double times[ITERATIONS];
     int subIterations = 0;
     for(int i=0;i<ITERATIONS;i++) {
@@ -90,7 +174,8 @@ public:
       fflush(stderr);
 #endif
     }
-    *subTime = uprv_getMeanTime(times,ITERATIONS,marginOfError);
+    uint32_t iterations = ITERATIONS;
+    *subTime = uprv_getMeanTime(times,&iterations,marginOfError);
     return subIterations;
   }
 public:
@@ -101,7 +186,17 @@ public:
 };
 
 void runTestOn(HowExpensiveTest &t) {
-  fprintf(stderr, "%s:%d: Running: %s\n", t.fFile, t.fLine, t.fName);
+  if(U_FAILURE(setupStatus)) return; // silently
+  const char *tn = t.getName();
+  if(testName!=NULL && testMatch(tn)) return; // skipped.
+  if(listmode) {
+    fprintf(stderr, "%s:%d:\t%s\n", t.fFile, t.fLine, t.getName());
+    testhit++;
+    return;
+  } else {
+    fprintf(stderr, "%s:%d: Running: %s\n", t.fFile, t.fLine, t.getName());
+    testhit++;
+  }
   double sieveTime = uprv_getSieveTime(NULL);
   double st;
   double me;
@@ -109,16 +204,20 @@ void runTestOn(HowExpensiveTest &t) {
   fflush(stdout);
   fflush(stderr);
   int32_t iter = t.runTests(&st,&me);
+  if(U_FAILURE(setupStatus)) {
+    fprintf(stderr, "Error in tests: %s\n", u_errorName(setupStatus));
+    return;
+  }
   fflush(stdout);
   fflush(stderr);
   
   double stn = st/sieveTime;
 
-  printf("%s\t%.9f\t%.9f +/- %.9f,  @ %d iter\n", t.fName,stn,st,me,iter);
+  printf("%s\t%.9f\t%.9f +/- %.9f,  @ %d iter\n", t.getName(),stn,st,me,iter);
 
   if(out!=NULL) {
     fprintf(out, "   <test name=\"%s\" standardizedTime=\"%f\" realDuration=\"%f\" marginOfError=\"%f\" iterations=\"%d\" />\n",
-            t.fName,stn,st,me,iter);
+            tn,stn,st,me,iter);
     fflush(out);
   }
 }
@@ -246,6 +345,34 @@ public:
 
 #define DO_AttrNumTest(p,n,x,a,v) { AttrNumTest t(p,n,x,__FILE__,__LINE__,a,v); runTestOn(t); }
 
+
+class NOXNumTest : public NumTest 
+{
+private:
+  UNumberFormatAttribute fAttr;
+  int32_t fAttrValue;
+  char name2[100];
+protected:
+  virtual const char *getClassName() {
+    sprintf(name2,"NOXNumTest:%d=%d", fAttr,fAttrValue);
+    return name2;
+  }
+public:
+  NOXNumTest(const char *pat, const char *num, double expect, const char *FILE, int LINE /*, UNumberFormatAttribute attr, int32_t newValue */) 
+    : NumTest(pat,num,expect,FILE,LINE) /* ,
+      fAttr(attr),
+      fAttrValue(newValue) */
+  {
+  }
+  virtual UNumberFormat* initFmt() {
+    UNumberFormat *fmt = NumTest::initFmt();
+    //unum_setAttribute(fmt, fAttr,fAttrValue);
+    return fmt;
+  }
+};
+
+#define DO_NOXNumTest(p,n,x) { NOXNumTest t(p,n,x,__FILE__,__LINE__); runTestOn(t); }
+
 #define DO_TripleNumTest(p,n,x) DO_AttrNumTest(p,n,x,UNUM_PARSE_ALL_INPUT,UNUM_YES) \
                                 DO_AttrNumTest(p,n,x,UNUM_PARSE_ALL_INPUT,UNUM_NO) \
                                 DO_AttrNumTest(p,n,x,UNUM_PARSE_ALL_INPUT,UNUM_MAYBE)
@@ -471,7 +598,7 @@ public:
     
   int32_t run() {
     int32_t trial;
-    int i;
+    int i=0;
     UnicodeString buf;
     if(U_SUCCESS(setupStatus)) {
       for(i=0;i<U_LOTS_OF_TIMES;i++){
@@ -488,18 +615,60 @@ public:
 
 // TODO: move, scope.
 static UChar pattern[] = { 0x23 }; // '#'
+static UChar strdot[] = { '2', '.', '0', 0 };
+static UChar strspc[] = { '2', ' ', 0 };
+static UChar strgrp[] = {'2',',','2','2','2', 0 };
+static UChar strbeng[] = {0x09E8,0x09E8,0x09E8,0x09E8, 0 };
 
 UNumberFormat *NumParseTest_fmt;
 
 // TODO: de-uglify.
 QuickTest(NumParseTest,{    static UChar pattern[] = { 0x23 };    NumParseTest_fmt = unum_open(UNUM_PATTERN_DECIMAL,         pattern,                    1,                    "en_US",                    0,                    &setupStatus);  },{    int32_t i;    static UChar str[] = { 0x31 };double val;    for(i=0;i<U_LOTS_OF_TIMES;i++) {      val=unum_parse(NumParseTest_fmt,str,1,NULL,&setupStatus);    }    return i;  },{unum_close(NumParseTest_fmt);})
 
+QuickTest(NumParseTestdot,{    static UChar pattern[] = { 0x23 };    NumParseTest_fmt = unum_open(UNUM_PATTERN_DECIMAL,         pattern,                    1,                    "en_US",                    0,                    &setupStatus);  },{    int32_t i;  double val;    for(i=0;i<U_LOTS_OF_TIMES;i++) {      val=unum_parse(NumParseTest_fmt,strdot,1,NULL,&setupStatus);    }    return i;  },{unum_close(NumParseTest_fmt);})
+QuickTest(NumParseTestspc,{    static UChar pattern[] = { 0x23 };    NumParseTest_fmt = unum_open(UNUM_PATTERN_DECIMAL,         pattern,                    1,                    "en_US",                    0,                    &setupStatus);  },{    int32_t i;    double val;    for(i=0;i<U_LOTS_OF_TIMES;i++) {      val=unum_parse(NumParseTest_fmt,strspc,1,NULL,&setupStatus);    }    return i;  },{unum_close(NumParseTest_fmt);})
+QuickTest(NumParseTestgrp,{    static UChar pattern[] = { 0x23 };    NumParseTest_fmt = unum_open(UNUM_PATTERN_DECIMAL,         pattern,                    1,                    "en_US",                    0,                    &setupStatus);  },{    int32_t i;    double val;    for(i=0;i<U_LOTS_OF_TIMES;i++) {      val=unum_parse(NumParseTest_fmt,strgrp,-1,NULL,&setupStatus);    }    return i;  },{unum_close(NumParseTest_fmt);})
+
+QuickTest(NumParseTestbeng,{    static UChar pattern[] = { 0x23 };    NumParseTest_fmt = unum_open(UNUM_PATTERN_DECIMAL,         pattern,                    1,                    "en_US",                    0,                    &setupStatus);  },{    int32_t i;    double val;    for(i=0;i<U_LOTS_OF_TIMES;i++) {      val=unum_parse(NumParseTest_fmt,strbeng,-1,NULL,&setupStatus);    }    return i;  },{unum_close(NumParseTest_fmt);})
+
+UDateFormat *DateFormatTest_fmt = NULL;
+UDate sometime = 100000000.0;
+UChar onekbuf[1024];
+const int32_t onekbuf_len = sizeof(onekbuf)/sizeof(onekbuf[0]);
+
+ 
+QuickTest(DateFormatTestBasic, \
+          { \
+            DateFormatTest_fmt = udat_open(UDAT_DEFAULT, UDAT_DEFAULT, NULL, NULL, -1, NULL, -1, &setupStatus); \
+          }, \
+          { \
+            int i; \
+            for(i=0;i<U_LOTS_OF_TIMES;i++)  \
+            { \
+              udat_format(DateFormatTest_fmt, sometime, onekbuf, onekbuf_len, NULL, &setupStatus); \
+            } \
+            return i; \
+          }, \
+          { \
+            udat_close(DateFormatTest_fmt); \
+          } \
+      )
+
 
 QuickTest(NullTest,{},{int j=U_LOTS_OF_TIMES;while(--j);return U_LOTS_OF_TIMES;},{})
+
+#if 0
+#include <time.h>
+
+QuickTest(RandomTest,{},{timespec ts; ts.tv_sec=rand()%4; int j=U_LOTS_OF_TIMES;while(--j) { ts.tv_nsec=100000+(rand()%10000)*1000000; nanosleep(&ts,NULL); return j;} return U_LOTS_OF_TIMES;},{})
+#endif
+
 OpenCloseTest(pattern,unum,open,{},(UNUM_PATTERN_DECIMAL,pattern,1,"en_US",0,&setupStatus),{})
 OpenCloseTest(default,unum,open,{},(UNUM_DEFAULT,NULL,-1,"en_US",0,&setupStatus),{})
+#if !UCONFIG_NO_CONVERSION
 #include "unicode/ucnv.h"
 OpenCloseTest(gb18030,ucnv,open,{},("gb18030",&setupStatus),{})
+#endif
 #include "unicode/ures.h"
 OpenCloseTest(root,ures,open,{},(NULL,"root",&setupStatus),{})
 
@@ -508,16 +677,52 @@ void runTests() {
     SieveTest t;
     runTestOn(t);
   }
+#if 0
+  {
+    RandomTest t;
+    runTestOn(t);
+  }
+#endif
   {
     NullTest t;
     runTestOn(t);
   }
+
+#ifndef SKIP_DATEFMT_TESTS
   {
-    NumParseTest t;
+    DateFormatTestBasic t;
     runTestOn(t);
   }
 #endif
 
+#ifndef SKIP_NUMPARSE_TESTS
+  {
+    // parse tests
+
+    DO_NumTest("#","0",0.0);
+    DO_NumTest("#","2.0",2.0);
+    DO_NumTest("#","2 ",2);
+    DO_NumTest("#","-2 ",-2);
+    DO_NumTest("+#","+2",2);
+    DO_NumTest("#,###.0","2222.0",2222.0);
+    DO_NumTest("#.0","1.000000000000000000000000000000000000000000000000000000000000000000000000000000",1.0);
+    DO_NumTest("#","123456",123456);
+
+    // attr
+#ifdef HAVE_UNUM_MAYBE
+    DO_AttrNumTest("#","0",0.0,UNUM_PARSE_ALL_INPUT,UNUM_YES);
+    DO_AttrNumTest("#","0",0.0,UNUM_PARSE_ALL_INPUT,UNUM_NO);
+    DO_AttrNumTest("#","0",0.0,UNUM_PARSE_ALL_INPUT,UNUM_MAYBE);
+    DO_TripleNumTest("#","2.0",2.0);
+    DO_AttrNumTest("#.0","1.000000000000000000000000000000000000000000000000000000000000000000000000000000",1.0,UNUM_PARSE_ALL_INPUT,UNUM_NO);
+#endif
+
+
+    //  {    NumParseTestgrp t;    runTestOn(t);  }
+    {    NumParseTestbeng t;    runTestOn(t);  }
+
+  }
+#endif
 
 #ifndef SKIP_NUMFORMAT_TESTS
   // format tests
@@ -538,7 +743,6 @@ void runTests() {
     DO_NumFmtStringPieceTest("#.0000","123.4560",spPI);
     DO_NumFmtStringPieceTest("#.00","123.46",spPI);
     
-#if 1
     DO_NumFmtTest("#","0",0.0);
     DO_NumFmtTest("#","12345",12345);
     DO_NumFmtTest("#","-2",-2);
@@ -551,22 +755,31 @@ void runTests() {
     DO_NumFmtInt64Test("#","-2",-2);
     DO_NumFmtInt64Test("+#","+2",2);
   }
-#endif
 
+#ifndef SKIP_NUM_OPEN_TEST
   {
     Test_unum_opendefault t;
     runTestOn(t);
   }
-  {
-    Test_ucnv_opengb18030 t;
-    runTestOn(t);
-  }
   {
     Test_unum_openpattern t;
     runTestOn(t);
   }
+#endif
+
+#endif /* skip numformat tests */
+#if !UCONFIG_NO_CONVERSION
+  {
+    Test_ucnv_opengb18030 t;
+    runTestOn(t);
+  }
+#endif
   {
     Test_ures_openroot t;
     runTestOn(t);
   }
+
+  if(testhit==0) {
+    fprintf(stderr, "ERROR: no tests matched.\n");
+  }
 }
